"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/ring/core' }"

"{ NameSpace: Smalltalk }"

RGClassDescriptionStrategy subclass:#RGMetaclassStrategy
	instanceVariableNames:'baseClass'
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Core-Kernel'
!

RGMetaclassStrategy comment:''
!

!RGMetaclassStrategy class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare


"
! !

!RGMetaclassStrategy methodsFor:'error handling'!

definition
    "Refer to the comment in ClassDescription|definition."

    ^ String streamContents: 
        [:strm |
        strm print: self.
        self owner hasTraitComposition ifTrue: [
            strm
                crtab;
                nextPutAll: 'uses: ';
                print: self traitComposition ].
        
        (self owner slotsNeedFullDefinition or: [ ClassDefinitionPrinter showFluidClassDefinition ])
            ifFalse: [  
                        strm
                            crtab;
                            nextPutAll: 'instanceVariableNames: ';
                            store: self owner instanceVariablesString]
            ifTrue: [ 
                        strm
                            crtab;
                            nextPutAll: 'slots: ';
                            nextPutAll: self owner slotDefinitionString]]
! !

!RGMetaclassStrategy methodsFor:'private - backend access'!

allClassVarNames

    self owner instanceSide allClassVarNames 
!

baseBehavior

    ^ self baseClass
!

baseClass

    ^ self backend forBehavior baseClassFor: self owner
!

baseClass: anRGClass

    self backend forBehavior setBaseClassFor: self owner to: anRGClass
!

bindingOf: varName
    
    ^ self baseClass bindingOf: varName
!

category

    ^ self baseClass category
!

classSide

    ^ self owner
!

classVarNames

    ^ self theNonMetaClass classVarNames
!

comment

    ^ self baseClass comment
!

defaultMetaClass

    "will be set correctly together with baseClass"
    ^ RGUnresolvedValue new default: nil.
!

initialize

    baseClass := RGUnresolvedValue recursive.
    
    super initialize.
    
    
!

innerBindingOf: varName

    ^self instanceSide innerBindingOf: varName
!

instanceSide
    ^ self baseClass 
!

isMeta

    ^true
!

isMetaclass

    ^true
!

makeResolved

    "try to set the correct name before resolving of it"
    ((self owner hasUnresolved: #name) and: [ self baseClass isRingResolved and: [ self baseClass hasResolvedName ] ]) ifTrue: [ 
        self owner pvtName: (self baseClass name, ' class') asSymbol.
         ].
    
    super makeResolved markAsRingResolved.
    
    baseClass := self baseClass.
!

metaclass

    "temporary" 
    ^ self
!

package

    ^ self baseClass package
!

pvtAsMetaclassTrait

    | newMetaclassTrait |
    
    (self owner resolvedProperties includesAnyOf: #(superclass traitComposition))
        ifTrue: [self error: 'This class cannot be converted to trait'].
        
    "TODO: check empty layout"
    newMetaclassTrait := RGMetaclassTrait unresolvedWithParent: self owner parent.
    newMetaclassTrait copyPropertiesFrom: self owner.

    (self owner hasResolvedName)
        ifTrue: [ newMetaclassTrait pvtName: ((self owner name withoutSuffix: ' class'), ' classTrait') asSymbol ].
    newMetaclassTrait pvtLocalMethods: self owner localMethods.
    
    ^ newMetaclassTrait
    
    
!

pvtBaseClass

    ^ baseClass value
!

pvtBaseClass: anRGClass

    self owner environment verifyOwnership: anRGClass.

    ^ baseClass := anRGClass
!

pvtResolvableProperties

    ^ super pvtResolvableProperties, { 
        #baseClass -> baseClass. 
   	}
    
    
!

referencedBehaviors

    ^ super referencedBehaviors, {self baseClass}
!

sibling

    ^ self baseClass.
!

theNonMetaClass
    ^self baseClass
! !

!RGMetaclassStrategy methodsFor:'testing'!

isMetaclassStrategy

    ^ true
!

storeName

    ^ 'RGMetaclass'
! !

!RGMetaclassStrategy methodsFor:'visiting'!

acceptVisitor: aVisitor 

    ^ aVisitor visitMetaclass: self owner
! !

