"{ Package: 'stx:goodies/ring/core' }"

"{ NameSpace: Smalltalk }"

RGBehaviorStrategy subclass:#RGTraitV2DescriptionStrategy
	instanceVariableNames:'layout'
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Core-Kernel'
!

RGTraitV2DescriptionStrategy comment:''
!

!RGTraitV2DescriptionStrategy methodsFor:'accessing'!

instanceVariables: aCollectionOfSymbols

    "specify instance variable names"

    (self owner hasUnresolved: #layout)
        ifTrue: [ self layout: (RGFixedLayout parent: self)].
        
    (self layout isPointerLayout not and: [ aCollectionOfSymbols isEmpty ])
        ifTrue: [ ^ self ].
        
    self layout cleanSlots.

    aCollectionOfSymbols do: [ :instVarName |
        layout addSlot: (RGInstanceVariableSlot named: instVarName asSymbol parent: self)].
!

instanceVariablesString
    "Answer a string of my instance variable names separated by spaces."

    ^String streamContents: [ :stream |
        self instVarNames 
            do: [ :each | stream nextPutAll: each ]
            separatedBy: [ stream space ] ]
! !

!RGTraitV2DescriptionStrategy methodsFor:'accessing - backend'!

layout

    ^ self backend forBehavior layoutFor: self owner
!

layout: anRGLayout

    self owner announceDefinitionChangeDuring: [ 
        self backend forBehavior setLayoutFor: self owner to: anRGLayout ].
! !

!RGTraitV2DescriptionStrategy methodsFor:'copying'!

copyForBehaviorDefinitionPostCopy

    super copyForBehaviorDefinitionPostCopy.
    layout := self layout copyForBehaviorDefinition.
    layout parent: self owner.
! !

!RGTraitV2DescriptionStrategy methodsFor:'initialization'!

classVariablesString
    "Answer a string of my class variable names separated by spaces."

    ^ String streamContents: [ :stream | 
        self owner classVarNames 
            do: [ :each | stream nextPutAll: each ]
            separatedBy: [ stream space ] ]
!

defaultLayout

    ^ self owner defaultFixedLayoutStubIn: self.
!

initialize

    super initialize.
    
    layout := self unresolvedValue: self defaultLayout. 
    self owner environment backend createUnresolvedClassGroupFor: self owner.
!

initializeUnresolved

    super initializeUnresolved.
    
    layout := self unresolvedValue: self defaultLayout.
!

instSize

    ^ self owner allInstVarNames size
!

instVarNames
    ^ self layout instVarNames
! !

!RGTraitV2DescriptionStrategy methodsFor:'printing'!

storeOn: aStream
    aStream
        nextPutAll: '(';
        nextPutAll: self storeName;
        nextPutAll: ' named: '.
    self owner  name storeOn: aStream.
    aStream nextPut: $)
! !

!RGTraitV2DescriptionStrategy methodsFor:'private - backend access'!

traitTransformationString		
        
    ^ self owner name 
! !

!RGTraitV2DescriptionStrategy methodsFor:'private - backend interface'!

isTrait

    ^true
!

makeResolved

    super makeResolved. 
    
    layout := self layout makeResolved markAsRingResolved.
!

pvtLayout

    ^ layout value
!

pvtLayout: anRGLayout

    self owner environment verifyOwnership: anRGLayout.

    ^ layout := anRGLayout
!

pvtResolvableProperties

    ^ super pvtResolvableProperties, { 
        #layout -> layout. 
     	}
! !

!RGTraitV2DescriptionStrategy methodsFor:'testing class hierarchy'!

kindOfSubclass
    "Answer a String that is the keyword that describes the receiver's kind of subclass,
     either a regular subclass, a variableSubclass, a variableByteSubclass,
     a variableWordSubclass, a weakSubclass, an ephemeronSubclass or an immediateSubclass.
     c.f. typeOfClass"
    ^self owner isVariable
        ifTrue:
            [self layout isBitsLayout
                ifTrue:
                    [self layout isByteLayout
                        ifTrue: [' variableByteSubclass: ']
                        ifFalse: [' variableWordSubclass: ']]
                ifFalse:
                    [self layout isWeakLayout
                        ifTrue: [' weakSubclass: ']
                        ifFalse: [' variableSubclass: ']]]
        ifFalse:
            [self layout isImmediateLayout 
                ifTrue: [' immediateSubclass: ']
                ifFalse:
                    [self layout isEphemeronLayout 
                        ifTrue: [' ephemeronSubclass: ']
                        ifFalse: [' subclass: ']]]
! !

!RGTraitV2DescriptionStrategy methodsFor:'utilities'!

allSlots

    | superclassSlots aSuperclass |
    aSuperclass := self owner superclass.
    superclassSlots := OrderedCollection new.
    ((aSuperclass == self owner) or: [aSuperclass isNil ]) ifFalse: [ 
         superclassSlots addAll: aSuperclass allSlots ].
    ^ (superclassSlots, self layout allSlots) asArray.
! !

!RGTraitV2DescriptionStrategy methodsFor:'variables'!

trait

    ^ self owner
! !

